// This program compares impulse responses and stochastic simulations with first and second order approximations for a small model without capital
// in which there are quadratic costs to adjusting prices. 
// In this model, there is no endogenous state variable, so pruning appears to make almost no difference.
// When the standard deviation of eps_A is sufficiently large standard, then the second order approximation 
// produces a substantially different result from linear approximations.
// For the model used here, see example #3 in http://faculty.wcas.northwestern.edu/~lchrist/course/optimalpolicyhandout.pdf

// Endogenous variables
var R h pie C a Rnat;

// Innovations
varexo eps_A;

// PARAMETERS
parameters nbeta chi beta epsil phi rho alpha nu pietarget;

beta = 0.99;
epsil = 5;
phi = 100;
rho = 0.9;
alpha = 1.5;
nbeta = beta;
//nu=1/(epsil-1);
nu=0;
chi=1;
pietarget=1.;
dbstop if error;

[A_SS,h_SS,R_SS,pie_SS,C_SS] = steadys(pietarget,beta,phi,nu,epsil,chi);

Rnat_SS = 1/(beta)-1;

model;

//Monetary Policy Rule
R = (pietarget/beta)-1+alpha*(pie(+1)-pietarget);

//Euler
1/(1+R) = beta*C/(C(+1)*pie(+1));

//new Keynesian Phillips Curve
((1+nu - (epsil/(epsil-1)))*(1-epsil)+epsil*(chi*h*C/exp(a)-1))*(1+phi*(pie-1)^2/2)-phi*(pie-1)*pie
   + beta*phi*(pie(+1)-1)*pie(+1);

// resource constraint
C*(1+(phi/2)*(pie-1)^2)-exp(a)*h;

//Technology Shocks

a = 1.85*a(-1)-0.855*a(-2)+eps_A;

1/(1+Rnat) = beta*exp(a-a(+1));
end;

initval;
R=R_SS; 
h=h_SS; 
pie=pie_SS; 
C=C_SS;
a=0; 
Rnat=Rnat_SS;
end;

shocks;
var eps_A;
stderr .01;
end;

steady;

//replic = INTEGER 
//irf = T

// first do impulse responses and stochastic simulation by linear approximation
options_.pruning = 0;
set_dynare_seed(1);

// option drop is used for computing second moments based on the simulations triggered by periods.
// stochastic simulations are also done to compute irf when order > 1 and drop is used here too.
// the first drop observations are dropped from the stochastic simulations when the second moments and/or irf's are computed.
// by default, drop = 100.
stoch_simul(order=1,irf=20,periods=100,nograph,drop=0) pie h R C a Rnat;

Clinear = C;
pielinear = pie;
hlinear = h;
Rlinear = R;
Alinear = a;
Rnatlinear = Rnat;

replinearC=C_eps_A;
replinearpie=pie_eps_A;
replinearh=h_eps_A;
replinearR=R_eps_A;
replinearRnat=Rnat_eps_A;
replineara=a_eps_A;

// Now do everything by 2nd order approximation, but no pruning 

set_dynare_seed(1);
// replic is a command used for irf. When order = 2, then the impulse responses are based on replic simulations.
// replic is also used in the stochastic simulations triggered by periods=xx. In this case, the program does
// replic stochastic simulations, each of length periods. replic = 50 by default. The results in oo_.endosimul are the
// last of these replic simulations. For simulations these replications are of course pointless, so replic = 1 here when periods > 0.

stoch_simul(order=2, periods=100,replic=1,nograph,drop=0) pie h R C a Rnat;
CNoPrunesim = C;
pieNoPrunesim = pie;
hNoPrunesim = h;
RNoPrunesim = R;
ANoPrunesim = a;
RnatNoPrunesim = Rnat;
stoch_simul(order=2,irf=20,replic=100,nograph) pie h R C a Rnat;
repNoPruneC=C_eps_A;
repNoPrunepie=pie_eps_A;
repNoPruneh=h_eps_A;
repNoPruneR=R_eps_A;
repNoPruneRnat=Rnat_eps_A;
repNoPrunea=a_eps_A;

// now do everything with order = 2 and pruning
set_dynare_seed(1);
stoch_simul(order=2,pruning,periods=100,replic=1,nograph,drop=0) pie h R C a Rnat;
CPrunesim = C;
piePrunesim = pie;
hPrunesim = h;
RPrunesim = R;
APrunesim = a;
RnatPrunesim = Rnat;
stoch_simul(order=2,irf=20,pruning,replic=100,nograph) pie h R C a Rnat;
repPruneC=C_eps_A;
repPrunepie=pie_eps_A;
repPruneh=h_eps_A;
repPruneR=R_eps_A;
repPruneRnat=Rnat_eps_A;
repPrunea=a_eps_A;

tt=1:length(repPrunea);
figure
ia=2;
ib=3;
subplot(ia,ib,1)
plot(tt,replinearC,tt,repNoPruneC,'o-',tt,repPruneC,'*-')
legend('linear','order=2, no pruning','order = 1, pruning')
title('consumption')
axis tight

subplot(ia,ib,2)
plot(tt,replinearpie,tt,repNoPrunepie,'o-',tt,repPrunepie,'*-')
title('inflation')
axis tight

subplot(ia,ib,3)
plot(tt,replinearh,tt,repNoPruneh,'o-',tt,repPruneh,'*-')
title('employment')
axis tight

subplot(ia,ib,4)
plot(tt,replinearR,tt,repNoPruneR,'o-',tt,repPruneR,'*-')
title('nominal rate')
axis tight

subplot(ia,ib,6)
plot(tt,replineara,tt,repNoPrunea,'o-',tt,repPrunea,'*-')
title('technology shock')
axis tight

subplot(ia,ib,5)
plot(tt,replinearRnat,tt,repNoPruneRnat,'o-',tt,repPruneRnat,'*-')
title('natural rate')
axis tight

figure

tt=1:length(C);
ia=2;
ib=3;
subplot(ia,ib,1)
tt=1:length(C);
plot(tt,Clinear,tt,CNoPrunesim,'o-',tt,CPrunesim,'*-')
legend('linear','order=2, no pruning','order = 1, pruning')
title('consumption')
axis tight

subplot(ia,ib,2)
plot(tt,pielinear,tt,pieNoPrunesim,'o-',tt,piePrunesim,'*-')
title('inflation')
axis tight

subplot(ia,ib,3)
plot(tt,hlinear,tt,hNoPrunesim,'o-',tt,hPrunesim,'*-')
title('employment')
axis tight

subplot(ia,ib,4)
plot(tt,Rlinear,tt,RNoPrunesim,'o-',tt,RPrunesim,'*-')
title('nominal rate')
axis tight

subplot(ia,ib,6)
plot(tt,Alinear,tt,ANoPrunesim,'o-',tt,APrunesim,'*-')
title('technology shock')
axis tight

subplot(ia,ib,5)
plot(tt,Rnatlinear,tt,RnatNoPrunesim,'o-',tt,RnatPrunesim,'*-')
title('natural rate')
axis tight